/********************************************************
 * Data Visualization - Pirates of the Caribbean        *
 *                      Checkout Hits                   *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author:  Benson Li                                   *
 * Supervisor: Rodger (Jieliang) Luo                           *
 *                                                      *
 * Purpose: Demonstrate how to make a simple 2D Matrix  *
 *          data visualization with different color     *
 *          schemes.                                    *
 *                                                      *
 * Usage:                                               *
 *          Press '1', '2', '3', '4' to apply different *
 *          color schemes.                              *
 *          Press '0' to use gray scale.                *
 ********************************************************/
// Variables that should change when changing dataset
// the function drawReleases() also needs to be changed

// the indexes are for the first non-zero entry in each of the movie columns
import controlP5.*;
ControlP5 cp5;
Slider abc;
PImage wallpaper;
PFont newfont;
PFont newfontSlider;
float time_value=0;
Slider s1;

MovieAttributes dead_mans_chest= new MovieAttributes();
int[] movie_release_dates_indexes = new int[]{9,17,66};
MovieAttributes black_pearl= new MovieAttributes();
MovieAttributes at_worlds_end= new MovieAttributes();
MovieAttributes strangers_tide= new MovieAttributes();
String[] titles;
///////////////////////////////////////////////////////////

// Objects
ArrayList<MovieAttributes> list_of_movies = new ArrayList<MovieAttributes>(); 
GraphAttributes graph= new GraphAttributes();

// Layout variables 
final float constantHorizontalMargin = 300;
final float constantVerticalMargin = 100;
final float constantVerticalMargin2 = 250;
// Setup variables
Table table;
//Color variables
PImage colorBar;
PImage colormap;

void setup(){
  /////////////////////LOADING TABLE AND SETUP VARIABLES////////////////////////
  //// Variables: graph.numRows, graph.numColumns 
  //// Objects: table 
  size(1920, 1080); // setup the size of the window
   //In Processing 3, the size() function must be the first line of code inside setup()
   //and needs to use numbers instead of variables. 
  colormap=loadImage("colormap.png");
  // Make the screen resizable.                     
  //surface.setResizable(true);
  //Put data into Processing table from csv
  table = loadTable("mat259_pirates_final.csv");
  // Get row and column information
  graph.numRows = table.getRowCount(); 
  graph.numColumns = table.getColumnCount();
  println("Rows: " + graph.numRows + " Columns: " + graph.numColumns);

  /////////////////////CREATING MOVIE DATA CONTAINERS////////////////////////
  //// Variables: graph.num_of_titles, graph.cellWidth, graph.cellHeight
  //// Effects: list_of_movies now contains the containers for each movie and

  list_of_movies.add(black_pearl);
  list_of_movies.add(dead_mans_chest);
  list_of_movies.add(at_worlds_end);
  list_of_movies.add(strangers_tide);

  graph.num_of_titles = list_of_movies.size();
  graph.cellWidth = (width - constantHorizontalMargin*2)/(graph.numRows-1);
  graph.cellHeight = (height - constantVerticalMargin-constantVerticalMargin2)/graph.num_of_titles;
  println("\n"+"Cell Width: "+ graph.cellWidth + "Cell Height: "+ graph.cellHeight);
  println(width);

  list_of_moviesInstantiator(graph.num_of_titles);
  // Objects
  // Also sets graph.hit_max
  list_of_moviesDataLoader(graph.num_of_titles, graph.cellHeight);

  println("Current window width and height: "+ width+"x"+height);
  /////////////////////MISC////////////////////////
  //// Variables: 
  //// Objects: graph.time_locations, titles, colorBar
  //// Effects: 
  // Extracting the titles
  titles = new String[graph.num_of_titles]; 
  for (int i=0; i<graph.num_of_titles; i++){
    titles[i] = table.getString(0, i+3);
  }
  println("Title should be black_pearl: "+ titles[0]);
  // Creating the x values for all the graphs
  graph.time_locations = new float [graph.numRows-1];
  for (int i = 0; i < graph.numRows-1; i++) {
    graph.time_locations[i] = constantHorizontalMargin+graph.cellWidth*i;
  } 
  println("Checking graph.time_locations x values: ");
  printArray(graph.time_locations);
  println("Last x value: "+graph.time_locations[graph.numRows-2]);
  println("width - constantHorizontalMargin: "+ (width-constantHorizontalMargin));
  colorBar = loadImage("colormap.png");
  newfont = createFont("Aileron-Bold.otf",28);
  newfontSlider = createFont("Aileron-Bold.otf",22);
  /////////////////////CREATING SLIDER////////////////////////
  //// Variables/Arrays: 
  //// Objects: 
  //// Effects: 
  cp5 = new ControlP5(this);
  s1=cp5.addSlider("Time Slider").setPosition(floor(constantHorizontalMargin),floor(constantVerticalMargin+4*graph.cellHeight)+constantVerticalMargin2/2)
    .setSize(floor(graph.cellWidth*(graph.numRows-1)),50)
    .setRange(0,graph.numRows-1)
    .setValue(0)
    .setColorBackground(color(135,206,250))
    .setColorForeground(color(204,0,100))
    .setColorActive(color(204,0,100))
    .setColorCaptionLabel(0)
    .setFont(newfontSlider);
  s1.addCallback(new CallbackListener(){
      public void controlEvent(CallbackEvent theEvent){
        if (theEvent.getAction()==ControlP5.ACTION_BROADCAST) {
          time_value = s1.getValue();
        }
      } 
    }
  );
  println("Current window width and height: "+ width+"x"+height);
  wallpaper = loadImage("background.png");

}

void draw(){
  background(wallpaper);
  //colorMode(HSB);
  //background(255,200,250);
  cp5.draw(); 
   //draw datapoints
  drawDataPoints(list_of_movies);
  //Draw graph lines
  graphLines(constantHorizontalMargin, constantVerticalMargin);
  drawVerticalLabels(constantHorizontalMargin, graph.cellHeight);
  drawHorizontalLabels(constantVerticalMargin, graph.cellHeight);
  drawTitles(constantVerticalMargin);
  drawReleases(constantHorizontalMargin, constantVerticalMargin);
  //draws colormap
  pushMatrix();
  translate(constantHorizontalMargin+graph.cellWidth*(graph.numRows-1)+constantHorizontalMargin/3,constantVerticalMargin+graph.cellHeight*3+140);
  rotate(-HALF_PI);
  image(colormap,0,0);
  popMatrix();
  drawSlider(time_value);
}

void Time(float value){
  time_value = floor(value);
}


// Utility Functions
void printArray(float[] one_d_array){
  for (int i = 0; i < 10; i++) {
    print(one_d_array[i]+", ");
  }
  println("End");
}
float maxArray(float[] one_d_array){
  float array_max = 0;
  for (int i = 0; i < one_d_array.length; i++) {
    if (array_max<one_d_array[i]){
      array_max=one_d_array[i];
    }
  } 
  return array_max;
}
//////////////////////////////////////

